<?xml version="1.0" encoding="iso-8859-1"?> 
<xsl:stylesheet 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	xmlns:xsd="http://www.w3.org/2001/XMLSchema"
	xmlns:rng="http://relaxng.org/ns/structure/1.0"
	version="1.0">

<xsl:output 
	method="xml" 
	indent="yes" 
	encoding="iso-8859-1"/>

<xsl:strip-space elements="*"/>

	<xsl:template match="/">
		<rng:grammar xmlns:rng="http://relaxng.org/ns/structure/1.0"
			xmlns:a="http://relaxng.org/ns/compatibility/annotations/1.0"
			
			
			xmlns:udt="urn:un:unece:uncefact:data:standard:UnqualifiedDataType:10"
			xmlns="fr:gouv:culture:archivesdefrance:seda:v1.0"
			xmlns:qdt="fr:gouv:culture:archivesdefrance:seda:v1.0:QualifiedDataType:1"
			xmlns:xsd="http://www.w3.org/2001/XMLSchema"
			xmlns:ccts="urn:un:unece:uncefact:documentation:standard:CoreComponentsTechnicalSpecification:2"
			ns="fr:gouv:culture:archivesdefrance:seda:v1.0"
			datatypeLibrary="http://www.w3.org/2001/XMLSchema-datatypes">

			<!-- element racine -->
			<rng:start combine="choice">
				<rng:ref name="ArchiveTransfer"/>
			</rng:start>
			<xsl:for-each select=".//xsd:element[@name='ArchiveTransfer']">
				<rng:define name="{@name}">
					<rng:element name="{@name}">
						<rng:ref name="{@name}_{generate-id()}"/>
					</rng:element>
				</rng:define>
			</xsl:for-each>

			<!-- definition de tous les types complexes -->
			<xsl:for-each select=".//xsd:element[./xsd:element]">
				<rng:define name="{@name}_{generate-id()}">
					<xsl:for-each select="xsd:element">
						<xsl:apply-templates select="."/>
					</xsl:for-each>
					<xsl:apply-templates select="xsd:attribute"/> 
				</rng:define>
			</xsl:for-each>
			<!-- definition de tous les types avec attribut -->
			<xsl:for-each select=".//xsd:element[not(xsd:element)][xsd:attribute]">
				<rng:define name="{@name}_{generate-id()}">
					<xsl:choose>
						<xsl:when test="@fixed">
							<rng:value><xsl:value-of select="@fixed"/></rng:value>
						</xsl:when>
						<xsl:when test="(@type = 'ArchiveType') 
							or (@type = 'ArchiveObjectType')
							or (@type = 'DocumentType')
							or (@type = 'ContentDescriptionType')
							or (@type = 'KeywordType')
							or (@type = 'AppraisalRuleType')
							or (@type = 'AccessRestrictionRuleType')
							or (@type = 'RelatedDataType')
							or (@type = 'CustodialHistoryType')
							or (@type = 'RelatedObjectReferenceType')
							or (@type = 'CommunicationType')
							or (@type = 'OrganizationType')
							or (@type = 'ContactType')
							or (@type = 'AddressType')
							or (@type = 'SignatureType')">
							<rng:ref name="anyElement"/>
						</xsl:when>
						<xsl:otherwise>
							<rng:data type="string"/>
						</xsl:otherwise>
					</xsl:choose>
					<xsl:apply-templates select="xsd:attribute"/>
				</rng:define>
			</xsl:for-each>
			
			<rng:define name="anyElement">
				<rng:zeroOrMore>
					<rng:element>
						<rng:anyName/>
						<rng:zeroOrMore>
							<rng:choice>
								<rng:attribute>
									<rng:anyName/>
								</rng:attribute>
								<rng:text/>
								<rng:ref name="anyElement"/>
							</rng:choice>
						</rng:zeroOrMore>
					</rng:element>
				</rng:zeroOrMore>
			</rng:define>
		</rng:grammar>
	</xsl:template>


	<!--
	***
	***	Traitement du contenu d'un element
	***
	-->
	<xsl:template name="call4definitions">
		<rng:element name="{@name}">
			<xsl:choose>
				<xsl:when test="xsd:element">
					<rng:ref name="{@name}_{generate-id()}"/>
				</xsl:when>
				<xsl:when test="xsd:attribute">
					<rng:ref name="{@name}_{generate-id()}"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:choose>
						<xsl:when test="@fixed">
							<rng:value><xsl:value-of select="@fixed"/></rng:value>
						</xsl:when>
						<xsl:when test="(@type = 'ArchiveType') 
							or (@type = 'ArchiveObjectType')
							or (@type = 'DocumentType')
							or (@type = 'ContentDescriptionType')
							or (@type = 'KeywordType')
							or (@type = 'AppraisalRuleType')
							or (@type = 'AccessRestrictionRuleType')
							or (@type = 'RelatedDataType')
							or (@type = 'CustodialHistoryType')
							or (@type = 'RelatedObjectReferenceType')
							or (@type = 'CommunicationType')
							or (@type = 'OrganizationType')
							or (@type = 'ContactType')
							or (@type = 'AddressType')
							or (@type = 'SignatureType')">
							<rng:ref name="anyElement"/>
						</xsl:when>
						<xsl:otherwise>
							<rng:data type="string"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:otherwise>
			</xsl:choose>
		</rng:element>
	</xsl:template>

	<!--
	***
	***	Traitement d'un element
	***
	-->
	<xsl:template match="xsd:element">
		<xsl:variable name="min">
			<xsl:choose>
				<xsl:when test="not(@minOccurs)">1</xsl:when>
				<xsl:otherwise><xsl:value-of select="@minOccurs"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="max">
			<xsl:choose>
				<xsl:when test="not(@maxOccurs)">
					<xsl:choose>
						<xsl:when test="($min = '0')">1</xsl:when>
						<xsl:otherwise><xsl:value-of select="$min"/></xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise><xsl:value-of select="@maxOccurs"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="($min = '0') and ($max = '1')">
				<rng:optional><xsl:call-template name="call4definitions"/></rng:optional>
			</xsl:when>
			<xsl:when test="($min = '0') and ($max = 'unbounded')">
				<rng:zeroOrMore><xsl:call-template name="call4definitions"/></rng:zeroOrMore>
			</xsl:when>
			<xsl:when test="($min = '1') and ($max = 'unbounded')">
				<rng:oneOrMore><xsl:call-template name="call4definitions"/></rng:oneOrMore>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="call4definitions"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!--
	***
	***	Traitement d'un attribut
	***
	-->
	<xsl:template match="xsd:attribute">
		<xsl:choose>
			<xsl:when test="(@use = 'prohibited')"/>
			<xsl:when test="(@use = 'optional') and not(@fixed)">
				<rng:optional>
					<rng:attribute name="{@name}">
						<rng:data type="string"/>
					</rng:attribute>
				</rng:optional>
			</xsl:when>
			<xsl:when test="(@use = 'optional') and (@fixed)">
				<rng:optional>
					<rng:attribute name="{@name}">
						<rng:value><xsl:value-of select="@fixed"/></rng:value>
					</rng:attribute>
				</rng:optional>
			</xsl:when>
			<xsl:when test="(@use = 'required') and not(@fixed)">
				<rng:attribute name="{@name}">
					<rng:data type="string"/>
				</rng:attribute>
			</xsl:when>
			<xsl:when test="(@use = 'required') and (@fixed)">
				<rng:attribute name="{@name}">
					<rng:value><xsl:value-of select="@fixed"/></rng:value>
				</rng:attribute>
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<!--
	***
	***	Traitement d'une annotation
	***
	-->
	<xsl:template match="xsd:annotation"/>

</xsl:stylesheet>